#include "character.h"
#include "core/renderer.h"
#include "world.h"

namespace rogue {
Character::Character(World &world, Tile location, CharacterData stats)
: data{stats}
, health{stats.health}
, location{location}
, world{world}
{}

void Character::act() {
	if(this->health < 0) {
		return;
	}
	// get motion from logic
	Tile target{this->data.logic(*this)};
	if(target == this->location) {
		return;
	}
	// check resulting tile
	TileData tile{world.query_tile(target)};
	// if character, deal damage
	if(tile.character != nullptr) {
		tile.character->deal_damage(this->data.damage);
	} else if(!tile.is_wall) { // if empty, move
		this->location = target;
	} // if wall, do nothing
}

void Character::draw() {
	if(this->health > 0) {
		Render::draw(this->location, this->data.sprite);
	} // else { // draw gore pile
}

bool Character::deal_damage(int damage) {
	return (this->health -= damage) <= 0;
}
Tile null_character_logic_function(Character &character) {
	return character.location;
}
}
