#ifndef ROGUE_WORLD_H
#define ROGUE_WORLD_H

#include <vector>
#include <SDL2/SDL_rect.h>
#include "character.h"
#include "core/roguedefs.h"

namespace rogue {
class Character;

struct TileData {
	Character *character{nullptr};
	bool is_wall{false};
};

struct Room {
	Directions hallway_paths{0};
	SDL_Rect rect{0, 0, 1, 1};
	void draw();
	bool tile_is_wall(Tile local_tile) const;
};

struct World {
	friend class WorldGenerator;
	~World() = default;
	void act();
	void render();
	Room &get_room(Chunk chunk);
	TileData query_tile(Tile tile);
private:
	int chunk_size{1};
	unsigned shear{0};
	std::vector<Room> rooms{};
	std::vector<Character> characters{};
private:
	World() = default;
};

class WorldGenerator {
public:
	WorldGenerator() = default;
	WorldGenerator &with_chunk_size(unsigned side_length);
	WorldGenerator &with_world_size(unsigned side_length);
	WorldGenerator &with_room_size(unsigned min_side_length, unsigned max_side_length);
	World generate();
private:
	unsigned chunk_side_length{0}, world_side_length{0};
	unsigned max_room_size{0}, min_room_size{0};
};
}

#endif // !ROGUE_WORLD_H
